@@ -4,7 +4,8 @@ from ..config import Config
 from ..entity.enums import QuestionType, QuestionMode, QuestionField
 from ..entity.exception import BfEngineException
 from .set_config import SetConfig
-
+from .langconv import Converter
+import random
 
 class QACaller(CallerBase):
     """
@@ -38,6 +39,9 @@ class QACaller(CallerBase):
         self.search_faq_json_question_list_url = Config.base_url + "/ssm/dac/info"
         self.search_faq_json_corpus_list_url = Config.base_url + "/ssm/dac/lq"
         self.search_faq_json_related_sq_list_url = Config.base_url + "/ssm/dac/sq/related_candidate"
+        self.operate_faq_category_url  = Config.base_url + "/ssm/dac/category"
+        self.get_faq_sub_category_url  = Config.base_url + "/ssm/dac/category/sub"
+        self.get_predict_qa_list_url = Config.base_url + "/qa/predict"
 
     def upload(self, data_type: QuestionType, data_model: QuestionMode, file_path: str) -> str:
         """
@@ -148,7 +152,7 @@ class QACaller(CallerBase):
             )
         data = {
             "sq": data["sq"],
-            "category_id": -1,
+            "category_id": int(data["category_id"]),
             "tag_id_list": tag_id_list,
             "answers": answers
         }
@@ -169,7 +173,7 @@ class QACaller(CallerBase):
             "records": [
                 {
                     "sq_id": data_id,
-                    "lq_list": list(map(lambda lq: {'lq': lq}, data["lq"]))
+                    "lq_list": list(map(lambda lq: {'lq': lq}, data["lq"] if "lq" in data.keys() else []))
                 }
             ]
         }
@@ -264,9 +268,6 @@ class QACaller(CallerBase):
 
     def qa_list(self, field: QuestionField) -> str:
         data = self.sq_list(field)
-        for item in data:
-            corpus = self.lq_list(str(item["id"]))
-            item["corpus"] = corpus
         return data
 
     def sq_list(self, field: QuestionField) -> str:
@@ -280,6 +281,9 @@ class QACaller(CallerBase):
         msg = str(resp["message"])
         if code != 0:
             raise BfEngineException(code=code, msg=msg)
+        for sq_data in resp["data"]:
+            sq_data['id'] = str(sq_data['id'])
+            sq_data['category_id'] = str(sq_data['category_id'])
         return resp["data"]
 
     def lq_list(self, qa_id: str):
@@ -367,7 +371,7 @@ class QACaller(CallerBase):
         resp = requests.put(self.upload_faq_json_lq_url, headers=self.header, json=params).json()
         code = int(resp["code"])
         msg = str(resp["message"])
-        if code != 0:
+        if code != 0 and msg!="语料重复":
             raise BfEngineException(code=code, msg=msg)
         return True
 
@@ -388,7 +392,32 @@ class QACaller(CallerBase):
                 }
             ]
         }
-        resp = requests.post(self.upload_faq_json_lq_url, headers=self.header, json=params).json()
+        resp = requests.post(self.upload_faq_json_lq_url, headers=self.header, json=params, timeout=30).json()
+        code = int(resp["code"])
+        msg = str(resp["message"])
+        if code != 0:
+            raise BfEngineException(code=code, msg=msg)
+        lq_id = ""
+        lq_list = self.lq_list(question_id)
+        for lq in lq_list:
+            if lq['content'] == str(corpus):
+                lq_id = lq['id']
+        return lq_id
+
+    def add_corpus_list(self, question_id, corpus_list):
+        """
+        增加语料
+        :return:
+        """
+        params = {
+            'records': [
+                {
+                    'sq_id': int(question_id),
+                    'lq_list': corpus_list
+                }
+            ]
+        }
+        resp = requests.post(self.upload_faq_json_lq_url, headers=self.header, json=params, timeout=30).json()
         code = int(resp["code"])
         msg = str(resp["message"])
         if code != 0:
@@ -419,13 +448,33 @@ class QACaller(CallerBase):
             raise BfEngineException(code=code, msg=msg)
         return True
 
+    def delete_corpus_list(self, question_id, corpus_id_list):
+        """
+        删除语料
+        :return:
+        """
+        params = {
+            'records': [
+                {
+                    'sq_id': int(question_id),
+                    'lq_list': corpus_id_list
+                }
+            ]
+        }
+        resp = requests.delete(self.upload_faq_json_lq_url, headers=self.header, json=params).json()
+        code = int(resp["code"])
+        msg = str(resp["message"])
+        if code != 0:
+            raise BfEngineException(code=code, msg=msg)
+        return True
+
     def query_corpus(self, question_id, keyword=None, page=None, size=None):
         url = self.upload_faq_json_lq_url + '/' + str(question_id)
         if keyword:
             url = url + "?keyword=" + keyword.strip()
         if page is not None and size is not None:
             url = url + "&page_size=" + size + "&page_index=" + page
-        resp = requests.get(url).json()
+        resp = requests.get(url,headers=self.header, json={}).json()
         code = int(resp["code"])
         msg = str(resp["message"])
         if code != 0:
@@ -438,13 +487,11 @@ class QACaller(CallerBase):
         :param question:
         :return:
         """
-        ## 目前昨天 先删掉，再添加
+        ## 问题调更新接口，语料删除重插
         question_id = data.get('id')
-        if question_id is None:
-            raise BfEngineException(code=-1, msg='missing required parameter: id')
-        self.delete_question(question_id)
-        question_id = self.upload_json_sq(data=data)
-        self.upload_json_lq(question_id, data)
+        data["id"] = int(data["id"])
+        data["category_id"] = int(data["category_id"])
+        requests.put(self.upload_faq_json_sq_url, headers=self.header, json=data).json()
         return question_id
 
     def delete_question(self, question_id):
@@ -487,6 +534,8 @@ class QACaller(CallerBase):
         新增答案
         :return:
         """
+        data["id"] = int(data["id"])
+        data["sq_id"] = int(data["sq_id"])
         resp = requests.put(self.upload_faq_json_answer_url, headers=self.header, json=data).json()
         code = int(resp["code"])
         msg = str(resp["message"])
@@ -507,4 +556,133 @@ class QACaller(CallerBase):
         msg = str(resp["message"])
         if code != 0:
             raise BfEngineException(code=code, msg=msg)
-        return True
+        return
+
+    def query_category_list(self):
+        resp = requests.get(self.operate_faq_category_url, headers=self.header,
+                            json={}).json()
+        code = int(resp["code"])
+        msg = str(resp["message"])
+        if code != 0:
+            raise BfEngineException(code=code, msg=msg)
+        return resp["data"]
+
+    def add_category(self,parent_id,name:str):
+        params = {
+            'parent_id': int(parent_id),
+            'name': name
+        }
+        resp = requests.post(self.operate_faq_category_url, headers=self.header, json=params).json()
+        code = int(resp["code"])
+        msg = str(resp["message"])
+        if code != 0:
+            raise BfEngineException(code=code, msg=msg)
+        return resp['data']['id']
+
+    def update_category(self,id,name:str):
+        params = {
+            'id': int(id),
+            'name': name
+        }
+        resp = requests.put(self.operate_faq_category_url, headers=self.header, json=params).json()
+        code = int(resp["code"])
+        msg = str(resp["message"])
+        if code != 0:
+            raise BfEngineException(code=code, msg=msg)
+        return resp['data']['id']
+
+    def delete_category(self,id):
+        params = {
+            'id': int(id)
+        }
+        resp = requests.delete(self.operate_faq_category_url, headers=self.header, json=params).json()
+        code = int(resp["code"])
+        msg = str(resp["message"])
+        if code != 0:
+            raise BfEngineException(code=code, msg=msg)
+        return
+
+    def get_similar_recomend_qa_list(self,user_inputs:str,type:str):
+        sq_result = []
+        converter = Converter('zh-hans')
+        params = {"Text":converter.convert(user_inputs).lower(),'UniqueId':str(random.randint(100,10000)),"IsRelease": False,'Robot':self.app_id}
+        print("params: "+str(params))
+        print("predict request url: "+self.get_predict_qa_list_url)
+        resp = requests.request("POST", self.get_predict_qa_list_url, headers=self.header, json=params).json()
+        code = int(resp["code"])
+        msg = str(resp["message"])
+        print(resp['data'])
+        if code != 0:
+           raise BfEngineException(code=code, msg=msg)
+        change_range = 0
+        if(type == "similar"):
+           change_range = 3
+        elif(type == "recommend"):
+             change_range = 25
+
+        for match_sq in resp['data']:
+            if match_sq["score"] > match_sq["threshold"]-change_range and len(sq_result)<3:
+                sq_result.append(match_sq)
+        return sq_result
+
+    def get_similar_qa_list(self,user_inputs:str,max_topn,category_ids,online: bool=True):
+        similar_sq_result = []
+        sq_list = [];
+        sq_map = {};
+        category_ids_list = self.get_all_sub_list(category_ids)
+        self.get_all_sq_under_input_folders(category_ids_list, sq_list, sq_map)
+        converter = Converter('zh-hans')
+        params = {"Text":converter.convert(user_inputs).lower(),'UniqueId':str(random.randint(100,10000)),"IsRelease": online,'Robot':self.app_id}
+        print("params: "+str(params))
+        print("predict request url: "+self.get_predict_qa_list_url)
+        resp = requests.request("POST", self.get_predict_qa_list_url, headers=self.header, json=params).json()
+        code = int(resp["code"])
+        msg = str(resp["message"])
+        print(resp['data'])
+        if code != 0:
+            raise BfEngineException(code=code, msg=msg)
+        for match_sq in resp['data']:
+            if match_sq['sqId'] in sq_list:
+               ans_list = [];
+               match_sq['category_id'] = str(sq_map.get(match_sq['sqId']))
+               ans_resp = requests.get(self.upload_faq_json_sq_url + "/" + str(match_sq['sqId'])+"?field=answer", headers=self.header, json={}).json()
+               code = int(ans_resp["code"])
+               msg = str(ans_resp["message"])
+               if code != 0:
+                  raise BfEngineException(code=code, msg=msg)
+               for ans in ans_resp['data']['answers']:
+                   ans_list.append(ans['answer'])
+               match_sq['answer'] = ans_list
+               similar_sq_result.append(match_sq)
+        topn = int(max_topn)
+        similar_sq_result = sorted(similar_sq_result, key=lambda x: x["score"], reverse=True)[:topn]
+        return similar_sq_result
+
+    def get_all_sq_under_input_folders(self, category_ids_list, sq_list, sq_map):
+        data = {"search": [{"keyword": "", "field": "sq"}],
+                "conditions": {"category": {"id_list": category_ids_list, "logical_op": "and"}}}
+        sq_resp = requests.post(self.search_faq_json_question_list_url, headers=self.header, json=data).json()
+        code = int(sq_resp["code"])
+        msg = str(sq_resp["message"])
+        if code != 0:
+            raise BfEngineException(code=code, msg=msg)
+        for sq in sq_resp['data']:
+            sq_list.append(sq['id'])
+            kv = {sq['id']: sq['category_id']}
+            sq_map.update(kv)
+
+    def get_all_sub_list(self, category_ids):
+        category_ids_list = []
+        for category_id in category_ids:
+            header = self.header
+            header.update({"folder_id": str(category_id)})
+            result = requests.get(self.get_faq_sub_category_url, headers=header, json={}).json()
+            category_sub_ids_list = result["data"]["idList"]
+            for category_sub_id in category_sub_ids_list:
+                if not category_ids_list.__contains__(category_sub_id):
+                    category_ids_list.append(category_sub_id)
+        return category_ids_list
+
+
+
+
