@@ -1,14 +1,16 @@
 from ..core.test import TestRunner
 from ..entity.answer import Answer
+from .set import Set
 
 
 class Module:
     """
     出话模块
     """
-    def __init__(self, app_id, name):
+    def __init__(self, app_id,name,set):
         self.app_id = app_id
         self.name = name
+        self.set = set
 
     def query(self, text: str, online: bool) -> Answer:
         """
