@@ -23,7 +23,7 @@ class QuestionAnswering(Module):
     def __init__(self, app_id,set):
         super().__init__(app_id, 'qa',set)
         self.caller = QACaller(app_id)
-    def train(self, data: str = None, question_path: str = None, corpus_path: str = None,append: bool = False):
+    def train(self, data: str = None, question_path: str = None, corpus_path: str = None,append: bool = False,show_progress:bool=True):
         """
         :param data:
         :param question_path: 标准问路径
@@ -43,7 +43,7 @@ class QuestionAnswering(Module):
                 self._upload_question(question_path,append)
                 if corpus_path:
                     self._upload_corpus(corpus_path,append)
-            self._train()
+            return self._train(show_progress)
         except BfEngineException as bfe:
             log.error(bfe)
             return False
@@ -53,6 +53,10 @@ class QuestionAnswering(Module):
             traceback.print_exc()
             return False
         return True
+    def train_status(self,train_id:str=None):
+        if not train_id:
+            raise BfEngineException(code=-1, msg="train_id is null")
+        return self.caller.train_status(train_id)
     def publish(self):
         """
         :发布
@@ -89,7 +93,7 @@ class QuestionAnswering(Module):
         : 查询标准问列表
         :return: 标准问列表
         """
-        return self.caller.qa_list(QuestionField.ALL)
+        return self.caller.qa_list(QuestionField.SQ)
     def question_list(self) -> str:
         """
         : 查询标准问列表
@@ -103,9 +107,11 @@ class QuestionAnswering(Module):
         :return: 标准问列表
         """
         return self.caller.lq_list(question_id)
-    def _train(self):
+    def _train(self,show_progress:bool=True):
         log.info('qa: start training')
         train_id = self.caller.train()
+        if not show_progress:
+            return train_id
         progress = 0
 
         with tqdm(total=100) as bar:
@@ -187,19 +193,20 @@ class QuestionAnswering(Module):
         except Exception as e:
             log.error(e)
 
-    def update_corpus(self, question_id, corpus_id):
+    def update_corpus(self, question_id, corpus_id, corpus):
         """
         更新语料
         :return:
         """
-        return self.caller.update_corpus(question_id, corpus_id)
+        return self.caller.update_corpus(question_id, corpus_id, corpus)
 
     def add_corpus(self, question_id, corpus):
         """
         增加语料
         :return:
         """
-        return self.caller.add_corpus(question_id, corpus)
+        lq_id = self.caller.add_corpus(question_id, corpus)
+        return lq_id
 
     def delete_corpus(self, question_id, corpus_id):
         """
@@ -233,4 +240,43 @@ class QuestionAnswering(Module):
         :param question_data:
         :return:
         """
-        return self.caller.update_question(question_data)
+        question_id = int(question_data["id"])
+        self.caller.update_question(question_data)
+        if("lq" in question_data.keys()):
+          lq_list = self.caller.query_corpus(question_id)
+          lq_id_list = []
+          for lq in lq_list:
+              lq_id_list.append(lq['id'])
+          self.caller.delete_corpus_list(question_id,lq_id_list)
+          lq_list = question_data.get("lq")
+          self.caller.add_corpus_list(question_id,lq_list)
+
+    def update_answer(selfs, answer_data: dict):
+        """
+        更新答案
+        """
+        return  selfs.caller.update_answer(answer_data)
+
+    def query_category_list(self) -> str:
+        return self.caller.query_category_list()
+
+    def add_category(self,parent_id,name:str):
+        return self.caller.add_category(parent_id,name)
+
+    def update_category(self,id,name:str):
+        return self.caller.update_category(id,name)
+
+    def delete_category(self,id):
+        return self.caller.delete_category(id)
+
+    def get_similar_qa_list(self,user_inputs:str,max_topn,category_ids,online: bool=True) -> str:
+        return self.caller.get_similar_qa_list(user_inputs,max_topn,category_ids,online)
+
+    def get_similar_recomend_qa_list(self,user_inputs:str,type:str) -> str:
+        """
+        获取相似问/推荐问
+        :param user_inputs 用户问
+        :param type similar:相似问，recommend:推荐问
+        """
+        return self.caller.get_similar_recomend_qa_list(user_inputs,type)
+
